/****************************************************************************/
/*              Red Flags and Red Herrings in the Execution Plans           */
/*                                                                          */
/*                         Dmitri V. Korotkevitch                           */
/*                        http://aboutsqlserver.com                         */
/*                          dk@aboutsqlserver.com                           */
/****************************************************************************/
/*						         Two Plans                                  */
/****************************************************************************/

set nocount on
go

use SQLServerInternals
go

if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where p.name = 'GetSalesByState' and s.name = 'dbo') drop proc dbo.GetSalesByState;
go

create proc dbo.GetSalesByState
(
	@State nvarchar(32), 
	@StartDate smalldatetime,
	@EndDate smalldatetime
)
as
	select @State, sum(o.Amount) as [Total Sales]
	from dbo.Customers c join dbo.Orders o on
		c.CustomerID = o.CustomerID
	where 
		c.CustomerState = @State and 
		o.OrderDate between @StartDate and @EndDate
	option (loop join, hash join); -- Excluding MERGE join to focus on LOOP and HASH
go

-- Compiling it for the @State = 'MA'
dbcc freeproccache
go
exec dbo.GetSalesByState @State = 'MA', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
go

-- Inefficient plan due to parameter-sniffing
set statistics io, time on
exec dbo.GetSalesByState @State = 'FL', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
set statistics io, time off
go

dbcc freeproccache
go

set statistics io, time on
exec dbo.GetSalesByState @State = 'FL', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
set statistics io, time off
go