/****************************************************************************/
/*              Red Flags and Red Herrings in the Execution Plans           */
/*                                                                          */
/*                         Dmitri V. Korotkevitch                           */
/*                        http://aboutsqlserver.com                         */
/*                          dk@aboutsqlserver.com                           */
/****************************************************************************/
/*                            Parameter Sniffing                            */
/****************************************************************************/
set nocount on
go

use SQLServerInternals
go

alter proc dbo.GetSalesByState
(
	@State nvarchar(32), 
	@StartDate smalldatetime,
	@EndDate smalldatetime
)
as
	select @State, sum(o.Amount) as [Total Sales]
	from dbo.Customers c join dbo.Orders o on
		c.CustomerID = o.CustomerID
	where 
		c.CustomerState = @State and 
		o.OrderDate between @StartDate and @EndDate
	option (recompile); -- Compiles most optimal plan for parameter values at cost of recompilations/cpu load
go

exec dbo.GetSalesByState @State = 'MA', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
exec dbo.GetSalesByState @State = 'FL', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
go

alter proc dbo.GetSalesByState
(
	@State nvarchar(32), 
	@StartDate smalldatetime,
	@EndDate smalldatetime
)
as
	select @State, sum(o.Amount) as [Total Sales]
	from dbo.Customers c join dbo.Orders o on
		c.CustomerID = o.CustomerID
	where 
		c.CustomerState = @State and 
		o.OrderDate between @StartDate and @EndDate
	option (optimize for unknown); -- most common statistics value
	-- option (optimize for (@State='FL')); 
go

exec dbo.GetSalesByState @State = 'MA', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
exec dbo.GetSalesByState @State = 'FL', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
go