/****************************************************************************/
/*              Red Flags and Red Herrings in the Execution Plans           */
/*                                                                          */
/*                         Dmitri V. Korotkevitch                           */
/*                        http://aboutsqlserver.com                         */
/*                          dk@aboutsqlserver.com                           */
/****************************************************************************/
/*                               Join Hints                                 */
/****************************************************************************/
set nocount on
go

use SQLServerInternals
go

if exists(select * from sys.views v join sys.schemas s on v.schema_id = s.schema_id where s.name = 'dbo' and v.name = 'vCustomersStateNumbers') drop view dbo.vCustomersStateNumbers;
go

-- Do not use indexed views on volatile OLTP tables

create view dbo.vCustomersStateNumbers
with schemabinding
as
	select c.CustomerState, count_big(*) as CustomerCount
	from dbo.Customers c
	group by c.CustomerState;
go

create unique clustered index IDX_vCustomersStateNumbers_CustomerState
on dbo.vCustomersStateNumbers(CustomerState);
go

/* Be very careful with Join hints. They also force order of the joins in the statement.
You need to know what you are doing - otherwise, you may end up with the worse plans than before */
alter proc dbo.GetSalesByState
(
	@State nvarchar(32), 
	@StartDate smalldatetime,
	@EndDate smalldatetime
)
as
	declare
		@CustCount int
		,@LoopJoinThreshold int = 100 -- for demo purposes. Can be thousands in prod

	-- You can use indexed view, calc columns, etc - as long as cost is not high
	select @CustCount = CustomerCount
	from dbo.vCustomersStateNumbers with (noexpand) -- hint to work in non-Enterprise Editions
	where CustomerState = @State;

	if isnull(@CustCount,0) <= @LoopJoinThreshold
		select @State, sum(o.Amount) as [Total Sales]
		from dbo.Customers c INNER LOOP JOIN dbo.Orders o on
			c.CustomerID = o.CustomerID
		where 
			c.CustomerState = @State and 
			o.OrderDate between @StartDate and @EndDate;
	else 
		select @State, sum(o.Amount) as [Total Sales]
		from dbo.Customers c INNER HASH JOIN dbo.Orders o on
			c.CustomerID = o.CustomerID
		where 
			c.CustomerState = @State and 
			o.OrderDate between @StartDate and @EndDate;
go

exec dbo.GetSalesByState @State = 'MA', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
exec dbo.GetSalesByState @State = 'FL', @StartDate = '2019-06-01', @EndDate = '2019-07-01';
go
