/****************************************************************************/
/*              Red Flags and Red Herrings in the Execution Plans           */
/*                                                                          */
/*                         Dmitri V. Korotkevitch                           */
/*                        http://aboutsqlserver.com                         */
/*                          dk@aboutsqlserver.com                           */
/****************************************************************************/
/*                           Optimizations (1)                              */
/****************************************************************************/
set nocount on
go

use SQLServerInternals
go

create index IDX_Orders_Amount on dbo.Orders(Amount);
go

set statistics io on

select c.CustomerName, o.OrderDate, o.OrderNum, o.Amount
from 
	dbo.Customers c join dbo.Orders o on
		c.CustomerID = o.CustomerID
order by 
	o.Amount
offset 10000 rows fetch next 10 rows only;


;with OrderIDs(OrderId)
as
(
	select o.OrderId
	from dbo.Orders o 
	order by o.Amount
	offset 10000 rows fetch next 10 rows only
)
select c.CustomerName, o.OrderDate, o.OrderNum, o.Amount
from 
	OrderIDs oid join dbo.Orders o on
		oid.OrderId = o.OrderID 
	join dbo.Customers c on
		c.CustomerID = o.CustomerID
order by 
	o.Amount;

set statistics io off
go


-- Run script from 08.2.Optimizations (2).sql to emulate blocking

-- Blocked
update dbo.Orders
set DeliveryInstructions = 'b'
where OrderNum = '1';

-- Good
;with OrderIDs(OrderId)
as
(
	select o.OrderId
	from dbo.Orders o with (nolock) -- no locks in read_committed_snapshot
	where o.OrderNum = '1'
)
update o
set o.DeliveryInstructions = 'b'
from OrderIds oid inner loop join dbo.Orders o 
	on oid.OrderId = o.OrderId;
